"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Command = void 0;
const parser_1 = require("./parser");
const creator_1 = require("./creator");
class Command {
    constructor(_a) {
        var { name } = _a, meta = __rest(_a, ["name"]);
        this.parameters = '';
        this.description = '';
        this.version = '';
        this.options = {};
        this.children = [];
        this.name = name;
        Object.assign(this, meta);
        for (const command of this.children)
            command.parent = this;
        this.addPreset();
    }
    static execute(command, args) {
        const { data, options } = parser_1.parseArguments(args);
        command.execute(options, ...data);
    }
    execute(options, ...data) {
        const command = this.children.find(({ name }) => name === data[0]);
        if (command instanceof Command) {
            command.execute(options, ...data.slice(1));
        }
        else if ('v' in options || 'version' in options) {
            console.log(this.version);
        }
        else if ('h' in options || 'help' in options) {
            this.showHelp();
        }
        else if (this.executor instanceof Function) {
            this.executor(options, ...data);
        }
        else {
            throw ReferenceError(`Unknown "${data[0]}" command`);
        }
    }
    addPreset() {
        const { name, options, children } = this, version = { description: 'show Version number' }, help = { description: 'show Help information' };
        Object.assign(this.options, Object.assign(Object.assign(Object.assign({}, (this.version && { v: version, version })), { h: help, help }), options));
        if (name !== 'help' &&
            children[0] &&
            !children.find(({ name }) => name === 'help'))
            children.push(new Command(Object.assign(Object.assign({ name: 'help' }, help), { executor: () => this.showHelp() })));
    }
    showHelp() {
        console.log(this + '');
    }
    *getParentNames() {
        let that = this;
        while ((that = that.parent))
            yield that.name;
    }
    toString() {
        const result = [
            [this.parameters, this.name, ...this.getParentNames()]
                .reverse()
                .join(' ')
                .trim(),
            this.description,
            ''
        ];
        result.push('Options:');
        result.push(this.toOptionString());
        if (this.children[0]) {
            result.push('Commands:');
            result.push(this.toChildrenString());
        }
        return result.join('\n');
    }
    toOptionString() {
        return creator_1.createTable(Object.entries(this.options)
            .sort(([A], [B]) => A.localeCompare(B))
            .map(([name, { parameters = '', description = '' }]) => [
            '',
            name[1] ? '--' + name : '-' + name,
            parameters,
            description
        ]));
    }
    toChildrenString() {
        return creator_1.createTable(this.children
            .sort(({ name: A }, { name: B }) => A.localeCompare(B))
            .map(({ name, parameters = '', description = '' }) => [
            '',
            name,
            parameters,
            description
        ]));
    }
}
exports.Command = Command;
